#include "Polynomial.h"
#include <stdio.h>

int Polynomial::degree()
{
    return coefficients.size() - 1;
}

void Polynomial::addRoot(Complex r)
{
    roots.push_back(r);
    coefficients.resize(roots.size() + 1);
}

void Polynomial::remRoot()
{
    roots.pop_back();
    coefficients.resize(roots.size() + 1);
}

void Polynomial::findCoefficients()
{
    std::vector<int> idx;
    idx.resize(roots.size() + 1);

    coefficients[0] = 1;

    for (int i = 1; i <= degree(); i++)
    {
        coefficients[i] = Complex(0, 0);
        int j;

        for (j = 0; j < i; j++)
            idx[j] = j;

        while(true)
        {
            Complex c = Complex(1, 0);

            for (j = 0; j < i; j++)
                c = c * (-roots[idx[j]]);
            coefficients[i] = coefficients[i] + c;

            j = i - 1;

            while(true)
            {
                idx[j]++;

                if (idx[j] == roots.size() - ((i-1) - j))
                {
                    j--;
                    if (j < 0)
                        break;
                }
                else
                {
                    for (int k = j + 1; k < i; k++)
                        idx[k] = idx[k - 1] + 1;
                    break;
                }
            }

            if (j < 0)
                break;
        }
    }
}

Complex Polynomial::evaluate(Complex z)
{
    Complex result = Complex(0, 0);
    Complex zn = Complex(1, 0);

    for (int i = degree(); i >= 0; i--)
    {
        result = result + zn * coefficients[i];
        zn = zn * z;
    }

    return result;
}

void Polynomial::derivative(Polynomial& result)
{
    result.coefficients.clear();

    for (int i = 0; i <= degree()-1; i++)
    {
        Complex coef = coefficients[i] * (degree() - i);
        result.coefficients.push_back(coef);
    }
}

void Polynomial::Print()
{
    printf("roots:\n");
    for (int i = 0; i < roots.size(); i++)
        printf("  %f + %f i\n", roots[i].re, roots[i].im);

    printf("coef:\n");
    for (int i = 0; i < coefficients.size(); i++)
        printf("  %f + %f i\n", coefficients[i].re, coefficients[i].im);
}
