#include "System.h"
#include "CSDL.h"

CSystem sys;

CSystem::CSystem()
{
    mQuitRequest = false;
    buttons = 0;
    buttonsDown = 0;
    buttonsUp = 0;
    mouseX = 0;
    mouseY = 0;
}

void CSystem::wait(Uint32 ms)
{
    sdl.Delay(ms);
}

bool CSystem::isQuitRequested()
{
    return mQuitRequest;
}

void CSystem::processEvents()
{
    buttonsDown = 0;
    buttonsUp = 0;

    SDL_Event event;

    while (sdl.PollEvent(&event) != 0)
    {
        if (event.type == SDL_QUIT)
        {
            mQuitRequest = true;
        }
        else if (event.type == SDL_MOUSEBUTTONDOWN)
        {
            if (event.button.button == SDL_BUTTON_LEFT)
            {
                buttonsDown |= BUT_LEFT;
                buttons |= BUT_LEFT;
            }
            else if (event.button.button == SDL_BUTTON_RIGHT)
            {
                buttonsDown |= BUT_RIGHT;
                buttons |= BUT_RIGHT;
            }

            mouseX = event.button.x;
            mouseY = event.button.y;
        }
        else if (event.type == SDL_MOUSEBUTTONUP)
        {
            if (event.button.button == SDL_BUTTON_LEFT)
            {
                buttonsUp |= BUT_LEFT;
                buttons &= ~BUT_LEFT;
            }
            else if (event.button.button == SDL_BUTTON_RIGHT)
            {
                buttonsUp |= BUT_RIGHT;
                buttons &= ~BUT_RIGHT;
            }
        }
        else if (event.type == SDL_MOUSEMOTION)
        {
            // avoid changing the coordinates if a button was pressed this frame
            if (buttonsDown == 0)
            {
                mouseX = event.motion.x;
                mouseY = event.motion.y;
            }
        }
    }
}
